@@ -287,7 +287,7 @@ formattedTicksAndLabels.Date <-
     if(!mat)
         z <- x[is.finite(x)] # override changes
     z <- z[z >= range[1] & z <= range[2]]
-    z <- structure(sort(unique(z)), class = "Date")
+    z <- structure(sort(unique(z)), class = "Date")    
     if (is.logical(labels))
         labels <- format.Date(z, format = format.posixt)
 ##     if (identical(labels, TRUE))
@@ -659,7 +659,7 @@ panel.axis <-
                          just = just,
                          gp = gp.text))
     }
-    return()
+    invisible()
 }
 
 
