@@ -77,15 +77,18 @@ panel.abline <-
                         ...)
         }
     }
-    
-    h <- as.numeric(h)
-    v <- as.numeric(v)
-    
-    for(i in seq(along = h))
-        grid.lines(y=rep(h[i],2), default.units="native", gp = gpar(col=col.line,lty=lty,lwd=lwd))
 
-    for(i in seq(along = v))
-        grid.lines(x=rep(v[i],2), default.units="native", gp = gpar(col=col.line,lty=lty,lwd=lwd))
+    if (length(h <- as.numeric(h)))
+        grid.segments(y0 = h, y1 = h, default.units="native",
+                      gp = gpar(col = col.line, lty = lty, lwd = lwd))
+    if (length(as.numeric(v)))
+        grid.segments(x0 = v, x1 = v, default.units="native",
+                      gp = gpar(col = col.line, lty = lty, lwd = lwd))
+
+##     for(i in seq(along = h))
+##         grid.lines(y=rep(h[i],2), default.units="native", gp = gpar(col=col.line,lty=lty,lwd=lwd))
+##     for(i in seq(along = v))
+##         grid.lines(x=rep(v[i],2), default.units="native", gp = gpar(col=col.line,lty=lty,lwd=lwd))
     
 }
 
@@ -354,7 +357,7 @@ prepanel.loess <-
 #     if (zero.line) abline(h=0, ...)
 #     panel.loess(x, y, span = span, family = family,
 #                 degree = degree, evaluation = evaluation, ...)
-#     panel.xyplot(x, ,y, ...)
+#     panel.xyplot(x, y, ...)
 # }
 ## base R function exists
 
@@ -364,7 +367,7 @@ prepanel.loess <-
 panel.superpose <-
     function(x, y = NULL, subscripts, groups,
              panel.groups = "panel.xyplot",
-             col,
+             col = NA,
              col.line = superpose.line$col,
              col.symbol = superpose.symbol$col,
              pch = superpose.symbol$pch,
@@ -376,16 +379,28 @@ panel.superpose <-
              lwd = superpose.line$lwd,
              alpha = superpose.symbol$alpha,
              type = 'p',
-             ...)
+             ...,
+             distribute.type = FALSE)
 {
-    if (!missing(type))
+    if (distribute.type)
+    {
+        ## This is a slightly different version of panel.superpose.  It
+        ## has an explicit type argument which behaves like other
+        ## graphical parameters, i.e., it is repeated to be as long as the
+        ## number of groups, and one used for each group.  This is the
+        ## default behaviour of panel.superpose in S-PLUS.
+
+        ## have.type <- FALSE
+        type <- as.list(type)
+    }
+    else 
     {
         ## this is something of a hack, needed because without this,
         ## grouped displays with 'g' %in% type may draw a grid
         ## (courtesy of panel.groups) for each group, overwriting
         ## earlier ones.
 
-        have.type <- TRUE
+        ## have.type <- TRUE
         type <- unique(type)
         wg <- match('g', type, nomatch = NA)
         if (!is.na(wg))
@@ -393,12 +408,12 @@ panel.superpose <-
             panel.grid(h = -1, v = -1)
             type <- type[-wg]
         }
+        type <- list(type)
     }
-    else have.type <- FALSE
     x <- as.numeric(x)
     if (!is.null(y)) y <- as.numeric(y)
 
-    if (length(x)>0)
+    if (length(x) > 0)
     {
         if (!missing(col))
         {
@@ -413,44 +428,50 @@ panel.superpose <-
             if (is.factor(groups)) levels(groups)
             else sort(unique(groups))
         nvals <- length(vals)
-        col.line <- rep(col.line, length=nvals)
-        col.symbol <- rep(col.symbol, length=nvals)
-        pch <- rep(pch, length=nvals)
-        lty <- rep(lty, length=nvals)
-        lwd <- rep(lwd, length=nvals)
-        alpha <- rep(alpha, length=nvals)
-        cex <- rep(cex, length=nvals)
-        font <- rep(font, length=nvals)
-        fontface <- rep(fontface, length=nvals)
-        fontfamily <- rep(fontfamily, length=nvals)
+        col <- rep(col, length = nvals)
+        col.line <- rep(col.line, length = nvals)
+        col.symbol <- rep(col.symbol, length = nvals)
+        pch <- rep(pch, length = nvals)
+        lty <- rep(lty, length = nvals)
+        lwd <- rep(lwd, length = nvals)
+        alpha <- rep(alpha, length = nvals)
+        cex <- rep(cex, length = nvals)
+        font <- rep(font, length = nvals)
+        fontface <- rep(fontface, length = nvals)
+        fontfamily <- rep(fontfamily, length = nvals)
+        type <- rep(type, length = nvals)
 
         panel.groups <- 
             if (is.function(panel.groups)) panel.groups
             else if (is.character(panel.groups)) get(panel.groups)
             else eval(panel.groups)
 
-        for (i in seq(along=vals))
+        subg <- groups[subscripts]
+        ok <- !is.na(subg)
+        for (i in seq(along = vals))
         {
-            id <- (groups[subscripts] == vals[i])
+            id <- ok & (subg == vals[i])
             if (any(id))
             {
                 args <-
-                    list(x=x[id],
+                    list(x = x[id],
                          ## groups = groups,
                          subscripts = subscripts[id],
                          pch = pch[i], cex = cex[i],
                          font = font[i],
                          fontface = fontface[i],
                          fontfamily = fontfamily[i],
+                         col = col[i],
                          col.line = col.line[i],
                          col.symbol = col.symbol[i],
                          lty = lty[i],
                          lwd = lwd[i],
                          alpha = alpha[i],
+                         type = type[[i]],
                          ...)
-                if (have.type) args$type <- type
+                ## if (have.type) args$type <- type
                 if (!is.null(y)) args$y <- y[id]
-                do.call("panel.groups", args)
+                do.call(panel.groups, args)
             }
         }
     }
@@ -458,97 +479,107 @@ panel.superpose <-
 
 
 
-
 panel.superpose.2 <-
-    function(x, y = NULL, subscripts, groups,
-             panel.groups = "panel.xyplot",
-             col,
-             col.line = superpose.line$col,
-             col.symbol = superpose.symbol$col,
-             pch = superpose.symbol$pch,
-             cex = superpose.symbol$cex, 
-             font = superpose.symbol$font, 
-             fontface = superpose.symbol$fontface, 
-             fontfamily = superpose.symbol$fontfamily, 
-             lty = superpose.line$lty,
-             lwd = superpose.line$lwd,
-             alpha = superpose.symbol$alpha,
-             type = "p",
-             ...)
+    function(..., distribute.type = TRUE)
 {
-
-    ## This is a slightly different version of panel.superpose.  It
-    ## has an explicit type argument which behaves like other
-    ## graphical parameters, i.e., it is repeated to be as long as the
-    ## number of groups, and one used for each group.  This is the
-    ## default behaviour of panel.superpose in S-PLUS.
-
-    ## Original version contributed by Neil Klepeis
-
-    type <- as.list(type)
-
-    x <- as.numeric(x)
-    if (!is.null(y)) y <- as.numeric(y)
-
-    if (length(x)>0)
-    {
-        if (!missing(col))
-        {
-            if (missing(col.line)) col.line <- col
-            if (missing(col.symbol)) col.symbol <- col
-        }
-
-        superpose.symbol <- trellis.par.get("superpose.symbol")
-        superpose.line <- trellis.par.get("superpose.line")
-
-        vals <-
-            if (is.factor(groups)) levels(groups)
-            else sort(unique(groups))
-        nvals <- length(vals)
-        col.line <- rep(col.line, length=nvals)
-        col.symbol <- rep(col.symbol, length=nvals)
-        pch <- rep(pch, length=nvals)
-        lty <- rep(lty, length=nvals)
-        lwd <- rep(lwd, length=nvals)
-        alpha <- rep(alpha, length=nvals)
-        cex <- rep(cex, length=nvals)
-        font <- rep(font, length=nvals)
-        fontface <- rep(fontface, length=nvals)
-        fontfamily <- rep(fontfamily, length=nvals)
-        type <- rep(type, length=nvals)
-
-        panel.groups <- 
-            if (is.function(panel.groups)) panel.groups
-            else if (is.character(panel.groups)) get(panel.groups)
-            else eval(panel.groups)
-
-        for (i in seq(along=vals))
-        {
-            id <- (groups[subscripts] == vals[i])
-            if (any(id))
-            {
-                args <- list(x=x[id],
-                             groups = groups,
-                             subscripts = subscripts[id],
-                             pch = pch[i], cex = cex[i],
-                             font = font[i],
-                             fontface = fontface[i],
-                             fontfamily = fontfamily[i],
-                             col.line = col.line[i],
-                             col.symbol = col.symbol[i],
-                             lty = lty[i],
-                             lwd = lwd[i],
-                             alpha = alpha[i],
-                             type = type[[i]], ...)
-                if (!is.null(y)) args$y <- y[id]
-
-                do.call("panel.groups", args)
-            }
-        }
-    }
+    panel.superpose(...,
+                    distribute.type = distribute.type)
 }
 
 
+## panel.superpose.2 <-
+##     function(x, y = NULL, subscripts, groups,
+##              panel.groups = "panel.xyplot",
+##              col,
+##              col.line = superpose.line$col,
+##              col.symbol = superpose.symbol$col,
+##              pch = superpose.symbol$pch,
+##              cex = superpose.symbol$cex, 
+##              font = superpose.symbol$font, 
+##              fontface = superpose.symbol$fontface, 
+##              fontfamily = superpose.symbol$fontfamily, 
+##              lty = superpose.line$lty,
+##              lwd = superpose.line$lwd,
+##              alpha = superpose.symbol$alpha,
+##              type = "p",
+##              ...)
+## {
+
+##     ## This is a slightly different version of panel.superpose.  It
+##     ## has an explicit type argument which behaves like other
+##     ## graphical parameters, i.e., it is repeated to be as long as the
+##     ## number of groups, and one used for each group.  This is the
+##     ## default behaviour of panel.superpose in S-PLUS.
+
+##     ## Original version contributed by Neil Klepeis
+
+##     type <- as.list(type)
+
+##     x <- as.numeric(x)
+##     if (!is.null(y)) y <- as.numeric(y)
+
+##     if (length(x)>0)
+##     {
+##         if (!missing(col))
+##         {
+##             if (missing(col.line)) col.line <- col
+##             if (missing(col.symbol)) col.symbol <- col
+##         }
+
+##         superpose.symbol <- trellis.par.get("superpose.symbol")
+##         superpose.line <- trellis.par.get("superpose.line")
+
+##         vals <-
+##             if (is.factor(groups)) levels(groups)
+##             else sort(unique(groups))
+##         nvals <- length(vals)
+##         col.line <- rep(col.line, length = nvals)
+##         col.symbol <- rep(col.symbol, length = nvals)
+##         pch <- rep(pch, length = nvals)
+##         lty <- rep(lty, length = nvals)
+##         lwd <- rep(lwd, length = nvals)
+##         alpha <- rep(alpha, length = nvals)
+##         cex <- rep(cex, length = nvals)
+##         font <- rep(font, length = nvals)
+##         fontface <- rep(fontface, length = nvals)
+##         fontfamily <- rep(fontfamily, length = nvals)
+##         type <- rep(type, length = nvals)
+
+##         panel.groups <- 
+##             if (is.function(panel.groups)) panel.groups
+##             else if (is.character(panel.groups)) get(panel.groups)
+##             else eval(panel.groups)
+
+##         subg <- groups[subscripts]
+##         ok <- !is.na(subg)
+##         for (i in seq(along = vals))
+##         {
+##             id <- ok & (subg == vals[i])
+##             if (any(id))
+##             {
+##                 args <-
+##                     list(x=x[id],
+##                          groups = groups,
+##                          subscripts = subscripts[id],
+##                          pch = pch[i], cex = cex[i],
+##                          font = font[i],
+##                          fontface = fontface[i],
+##                          fontfamily = fontfamily[i],
+##                          col.line = col.line[i],
+##                          col.symbol = col.symbol[i],
+##                          lty = lty[i],
+##                          lwd = lwd[i],
+##                          alpha = alpha[i],
+##                          type = type[[i]], ...)
+##                 if (!is.null(y)) args$y <- y[id]
+
+##                 do.call(panel.groups, args)
+##             }
+##         }
+##     }
+## }
+
+
 
 
 
@@ -669,4 +700,3 @@ panel.mathdensity <-
     panel.lines(x = x, y = y, col = col.line, lwd = lwd, lty = lty, ...)
 }
 
-
