@@ -176,19 +176,20 @@ panel.parallel <-
 
 
 
-parallel <- function(x, ...)
-{
-    ocall <- match.call()
-    formula <- ocall$formula
-    if (!is.null(formula))
-    {
-        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
-        ocall$formula <- NULL
-        if (is.null(ocall$x)) ocall$x <- formula
-        eval(ocall, parent.frame())
-    }
-    else UseMethod("parallel")
-}
+parallel <- function(x, data, ...) UseMethod("parallel")
+
+## {
+##     ocall <- match.call()
+##     formula <- ocall$formula
+##     if (!is.null(formula))
+##     {
+##         warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+##         ocall$formula <- NULL
+##         if (!("x" %in% names(ocall))) ocall$x <- formula else warning("'formula' overridden by 'x'")
+##         eval(ocall, parent.frame())
+##     }
+##     else UseMethod("parallel")
+## }
 
 
 parallel.data.frame <-
@@ -200,7 +201,7 @@ parallel.data.frame <-
     ccall$data <- list(x = x)
     ccall$x <- ~x
     ccall[[1]] <- as.name("parallel")
-    ans <- eval(ccall, parent.frame())
+    ans <- eval.parent(ccall)
     ans$call <- ocall
     ans
 }
@@ -230,19 +231,20 @@ parallel.formula <-
                        at = 1:ncol(x), labels = colnames(x))),
              subset = TRUE)
 {
+    formula <- x
 
-    ## dots <- eval(substitute(list(...)), data, parent.frame())
+    ## dots <- eval(substitute(list(...)), data, environment(formula))
     dots <- list(...)
 
-    groups <- eval(substitute(groups), data, parent.frame())
-    subset <- eval(substitute(subset), data, parent.frame())
+    groups <- eval(substitute(groups), data, environment(formula))
+    subset <- eval(substitute(subset), data, environment(formula))
 
     ## Step 1: Evaluate x, y, etc. and do some preprocessing
     
     ## right.name <- deparse(substitute(x))
-    ## x <- eval(substitute(x), data, parent.frame())
+    ## x <- eval(substitute(x), data, environment(formula))
     form <-
-        latticeParseFormula(x, data,
+        latticeParseFormula(formula, data,
                             subset = subset, groups = groups,
                             multiple = FALSE,
                             outer = FALSE, subscripts = TRUE,
@@ -269,30 +271,32 @@ parallel.formula <-
 
 
     cond <- form$condition
-    number.of.cond <- length(cond)
+    ## number.of.cond <- length(cond)
     x <- as.data.frame(form$right)
 
-    if (number.of.cond == 0) {
+    if (length(cond) == 0) {
         strip <- FALSE
         cond <- list(as.factor(rep(1, nrow(x))))
-        number.of.cond <- 1
+        ## number.of.cond <- 1
     }
 
     if (!missing(varnames)) colnames(x) <-
-        eval(substitute(varnames), data, parent.frame())
+        eval(substitute(varnames), data, environment(formula))
 
     ## create a skeleton trellis object with the
     ## less complicated components:
 
-    foo <- do.call("trellis.skeleton",
-                   c(list(cond = cond,
-                          aspect = aspect,
-                          between = between,
-                          strip = strip,
-                          panel = panel,
-                          xlab = xlab,
-                          ylab = ylab,
-                          xlab.default = "Parallel Coordinate Plot"), dots))
+    foo <-
+        do.call("trellis.skeleton",
+                c(list(formula = formula, 
+                       cond = cond,
+                       aspect = aspect,
+                       between = between,
+                       strip = strip,
+                       panel = panel,
+                       xlab = xlab,
+                       ylab = ylab,
+                       xlab.default = "Parallel Coordinate Plot"), dots))
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
@@ -302,40 +306,21 @@ parallel.formula <-
 
     ## overriding at and labels, maybe not necessary
     
-    ## scales <- eval(substitute(scales), data, parent.frame())
-
     if (is.character(scales)) scales <- list(relation = scales)
-
-#     if (is.null(scales$alternating)) {
-#         if (is.null(scales$y)) scales$y <- list(alternating = FALSE)
-#         else if (is.null(scales$y$alternating)) scales$y$alternating <- FALSE
-#         ## bug if y="free" but who cares
-#     }
-
-
-
-
     scales <- updateList(default.scales, scales)
-    foo <- c(foo, 
-             do.call("construct.scales", scales))
-
-#     ## forcing this for now. Shouldn't be too hard to give more
-#     ## control to the user
-
-#     ##foo$x.scales$at <- c(0,1)
-#     ##foo$x.scales$labels <- c("Min","Max")
-#     ##foo$y.scales$at <- 1:ncol(x)
-#     ##foo$y.scales$labels <- 
+    foo <- c(foo, do.call("construct.scales", scales))
     
     ## Step 3: Decide if limits were specified in call:
 
     have.xlim <- !missing(xlim)
-    if (!is.null(foo$x.scales$limit)) {
+    if (!is.null(foo$x.scales$limit))
+    {
         have.xlim <- TRUE
         xlim <- foo$x.scales$limit
     }
     have.ylim <- !missing(ylim)
-    if (!is.null(foo$y.scales$limit)) {
+    if (!is.null(foo$y.scales$limit))
+    {
         have.ylim <- TRUE
         ylim <- foo$y.scales$limit
     }
@@ -366,48 +351,34 @@ parallel.formula <-
 
     cond.max.level <- unlist(lapply(cond, nlevels))
 
-
-    id.na <- FALSE
-    for (j in 1:ncol(x))
-        id.na <- id.na | is.na(x[,j])
-    for (var in cond)
-        id.na <- id.na | is.na(var)
-    if (!any(!id.na)) stop("nothing to draw")
-    ## Nothing simpler ?
-
-
-    ## Step 6: Evaluate layout, panel.args.common and panel.args
-
+    ## Step 6: Determine packets
 
     foo$panel.args.common <-
         c(list(z = x, groups = groups), dots)
 
 
-    nplots <- prod(cond.max.level)
-    if (nplots != prod(sapply(foo$condlevels, length))) stop("mismatch")
-    foo$panel.args <- vector(mode = "list", length = nplots)
+    npackets <- prod(cond.max.level)
+    if (npackets != prod(sapply(foo$condlevels, length))) 
+        stop("mismatch in number of packets")
+    foo$panel.args <- vector(mode = "list", length = npackets)
 
 
-    cond.current.level <- rep(1, number.of.cond)
-
-
-    for (panel.number in seq(length = nplots))
+    foo$packet.sizes <- numeric(npackets)
+    if (npackets > 1)
     {
+        dim(foo$packet.sizes) <- sapply(foo$condlevels, length)
+        dimnames(foo$packet.sizes) <- lapply(foo$condlevels, as.character)
+    }
 
-        id <- !id.na
-        for(i in 1:number.of.cond)
-        {
-            var <- cond[[i]]
-            id <- id &
-            if (is.shingle(var))
-                ((var >=
-                  levels(var)[[cond.current.level[i]]][1])
-                 & (var <=
-                    levels(var)[[cond.current.level[i]]][2]))
-            else (as.numeric(var) == cond.current.level[i])
-        }
+    cond.current.level <- rep(1, length(cond))
 
-        foo$panel.args[[panel.number]] <-
+
+    for (packet.number in seq(length = npackets))
+    {
+        id <- compute.packet(cond, cond.current.level)
+        foo$packet.sizes[packet.number] <- sum(id)
+
+        foo$panel.args[[packet.number]] <-
             list(subscripts = subscr[id])
 
         cond.current.level <-
@@ -415,19 +386,20 @@ parallel.formula <-
                     cond.max.level)
     }
 
-    more.comp <- c(limits.and.aspect(prepanel.default.parallel,
-                                     prepanel = prepanel, 
-                                     have.xlim = have.xlim, xlim = xlim, 
-                                     have.ylim = have.ylim, ylim = ylim, 
-                                     x.relation = foo$x.scales$relation,
-                                     y.relation = foo$y.scales$relation,
-                                     panel.args.common = foo$panel.args.common,
-                                     panel.args = foo$panel.args,
-                                     aspect = aspect,
-                                     nplots = nplots,
-                                     x.axs = foo$x.scales$axs,
-                                     y.axs = foo$y.scales$axs),
-                   cond.orders(foo))
+    more.comp <-
+        c(limits.and.aspect(prepanel.default.parallel,
+                            prepanel = prepanel, 
+                            have.xlim = have.xlim, xlim = xlim, 
+                            have.ylim = have.ylim, ylim = ylim, 
+                            x.relation = foo$x.scales$relation,
+                            y.relation = foo$y.scales$relation,
+                            panel.args.common = foo$panel.args.common,
+                            panel.args = foo$panel.args,
+                            aspect = aspect,
+                            npackets = npackets,
+                            x.axs = foo$x.scales$axs,
+                            y.axs = foo$y.scales$axs),
+          cond.orders(foo))
     foo[names(more.comp)] <- more.comp
 
     class(foo) <- "trellis"
